/**
 * @file   cpath.c
 * @author David Bremner
 * @date   Wed Apr 17 14:44:32 2002
 * 
 * @brief Generate canonical (combinatorially distinct) path complexes
 * (as label sequences). Cannibalized from the old genpath.c
 *	   
 * 
 *  
*/

#include <stdlib.h>
#include <stdio.h>
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "oms_error.h"
#include "oms_types.h"

static void allocate_memory(natural_t,natural_t);
static void push(natural_t,natural_t);
static void pop(natural_t);
static int empty(natural_t);
static natural_t top(natural_t);


/* globals */

static natural_t dim,steps;  
natural_t max_sym;	/* the highest index used */
array_index_t cur_col;

#define min(a,b) ( (a) < (b) ? (a) : (b))
natural_t *count;

int *label;

extern 
void cpath_init(int d, int k){
  dim=d;
  steps=k;

  allocate_memory(d,k);

  max_sym=0;
  cur_col=0;
  push(0,0);

}

 
extern void cpath_next(){

  int s;

  for (cur_col++ ; cur_col<=steps; cur_col++)
    {

      if (dim-max_sym < steps-cur_col+1) {
	for (s=min(max_sym+1,dim); s>0 ; s--){ 
	  if (s!=top(cur_col-1)) {
	    push(s,cur_col);
	  }
	}
      } else {
	push(max_sym+1,cur_col);
      }
      
      if (top(cur_col)>max_sym && max_sym<= dim)
	max_sym++;	/* keep track of symbols used */
      count[top(cur_col)]++;
    }
}



extern 
bool cpath_more(){
  return(!empty(1));
}



 extern void  cpath_remove(){
   pop(--cur_col);
   
   while(empty(cur_col) && cur_col>1){
     pop(--cur_col);
   }
 }


natural_t *s_top;      /* array 1..steps of stack indexes */
natural_t **stack;   /* array of stacks */




void allocate_memory(natural_t d, natural_t k)
{
  natural_t i;

  /* for symmetry checking */
  label=calloc(d+1,sizeof(natural_t));

  count=calloc(d+1,sizeof(natural_t));
  s_top=calloc(k+1,sizeof(natural_t));
  stack=calloc(k+1,sizeof(natural_t*));
  for (i=0; i<= k; i++){
    s_top[i]=0;
    stack[i]=calloc(d,sizeof(natural_t));
  }
}

#define INDEX_CHECK(j) if (j<0 || j>steps) err_fatal(HERE,"index out of range");

static int empty(natural_t j){

  INDEX_CHECK(j);

  return(s_top[j] == 0);
}

static void pop(natural_t j){
  INDEX_CHECK(j);

  if (empty(j)) err_fatal(HERE,"attempt to pop empty stack");

  
  count[top(j)]--;
  if (count[max_sym]==0  && (max_sym>0) ) max_sym--;

  s_top[j]--;
  if (!empty(j)) 
    {
      /* we uncovered something */
      count[top(j)]++;
      if (top(j)>max_sym && max_sym<= dim) max_sym++;	/* keep track of symbols used */
    }
}

static void push(natural_t s,natural_t j){
    INDEX_CHECK(j);

    if (s_top[j]>=dim) err_fatal(HERE,"stack overflow");

    stack[j][s_top[j]++]=s;
    

    
}

static natural_t top(natural_t j){
  INDEX_CHECK(j);

  if (empty(j)) err_fatal(HERE,"empty stack has no top element");

  return(stack[j][s_top[j]-1]);
}

extern bool cpath_check_symmetric(){
  int i;
  int counter=1;

  for (i=0; i<=dim; i++){
    label[i]=0;
  }

  for (i=steps; i>=1; i--){
    if (label[top(i)]==0){
      label[top(i)]=counter++;
    } 
    if (top(steps-i+1)!=label[top(i)])
      return 0;
  }
  return 1;
}

extern void cpath_output( int *label){
  natural_t j,p;
  for (j=1; j<= steps ; j++){
    if (label){
      p=label[top(j)];
    } else {
      p=top(j);
    }
    printf("%d\t", p);
  }
  printf("\n");

}

