/**
 * @file   subset.c
 * @author David Bremner <bremner@unb.ca>
 * @date   Sat Jun 15 16:23:32 2002
 * 
 * @brief  compute a random subset of a matroid polytope description
 * 
 * 
 */


#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include "parser.h"
#include "gparams.h"
#include "combinat.h"
#include "oms_error.h"
#include "output.h"
#include "uni.h"

#include "basis_equation.h"
   
static void usage(char ** argv){
  fprintf(stderr,"usage: %s: [options] n n n\n",argv[0]);
  fprintf(stderr,"\nInput/Output:\n");
  fprintf(stderr,"-i <file>\ttake input from <file>\n");
  fprintf(stderr,"-o <file>\twrite input to <file>\n");
  fprintf(stderr,"\nSubsetting:\n");
  fprintf(stderr,"-C <n>\tzero <n> entries in the chirotope\n");
  fprintf(stderr,"-B <n>\tzero <n> entries in the boundary\n");
  fprintf(stderr,"n n n\tpreserve the listed boundary facets");
}


int main(int argc, char** argv){
  int c;
  int n,r;
  int num_bases;
  int *chi_modified,*bdy_modified;

  int i;
  int facets;
  sa_array_p chi,bdy;

  int bdy_zero=-1,chi_zero=-1;
  
  params_p params=malloc(sizeof(params_t));

  bool *bdy_keep=NULL;

  while((c=getopt(argc,argv,"i:o:C:B:b:"))!=-1){
    switch(c){
    case 'i':
      freopen(optarg,"r",stdin);
      assert(stdin);
      break;
    case 'o':
      freopen(optarg,"w",stdout);
      assert(stdout);
      break;
    case 'C':
      chi_zero=atoi(optarg);
      break;
    case 'B':
      bdy_zero=atoi(optarg);
      break;
    default:
      usage(argv);
      exit(1);

    }
  }
  
  parse_input(params);

  n=params->n;
  r=params->r;

  num_bases=binomial(n,r);
  facets=binomial(n,r-1);

  bdy_keep=calloc(facets,sizeof(bool));

  chi=sa_new(num_bases,0);
  bdy=sa_new(facets,0);


  for (i=0; i< facets; i++){
    sa_init(bdy,i,0);
    bdy_keep[i]=0;
  }

  while(optind<argc){
    bdy_keep[atoi(argv[optind++])-1]=1;
  }

  for (i=0; i< num_bases; i++){
    sa_init(chi,i,0);
  }

  for (i=0 ; i<parser.boundary_count; i++){
    if (parser.boundary_list[i]>0){
      sa_init(bdy,parser.boundary_list[i]-1,1);
    } else {
      sa_init(bdy,-1*(parser.boundary_list[i])-1,-1);      
    }
  }

  for (i=0; i<parser.chirotope_count; i++){
    if (parser.chirotope_list[i]>0){
      sa_init(chi,parser.chirotope_list[i]-1,1);
    } else {
      sa_init(chi,-1*parser.chirotope_list[i]-1,-1);      
    }
  }

  chi_modified=calloc(num_bases,sizeof(int));
  bdy_modified=calloc(facets,sizeof(int));
  while (chi_zero > 0){
    i=uni()*num_bases;

    if (!chi_modified[i] && sa_read(chi,i)){
      sa_init(chi,i,0);
      chi_zero--;
      chi_modified[i]=1;
    }
  }

  while (bdy_zero > 0){
    i=uni()*facets;

    if (!bdy_keep[i] && !bdy_modified[i] && sa_read(bdy,i)){
      sa_init(bdy,i,0);
      bdy_zero--;
      bdy_modified[i]=1;
    }
  }
  output_verbose(params,chi,bdy);
  return(0);
}
